@@ -5,7 +5,7 @@ Arithmetic is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2022 Free Software Foundation, Inc.
+Copyright (C) 2015-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -37,11 +37,13 @@ enum arithmetic_prog_operators
   ARITHMETIC_OP_FILTER_SIGCLIP_MEDIAN,
   ARITHMETIC_OP_ERODE,
   ARITHMETIC_OP_DILATE,
+  ARITHMETIC_OP_NUMBER_NEIGHBORS,
   ARITHMETIC_OP_CONNECTED_COMPONENTS,
   ARITHMETIC_OP_FILL_HOLES,
   ARITHMETIC_OP_INVERT,
   ARITHMETIC_OP_INTERPOLATE_MINNGB,
   ARITHMETIC_OP_INTERPOLATE_MAXNGB,
+  ARITHMETIC_OP_INTERPOLATE_MEANNGB,
   ARITHMETIC_OP_INTERPOLATE_MEDIANNGB,
   ARITHMETIC_OP_INTERPOLATE_MINOFREGION,
   ARITHMETIC_OP_INTERPOLATE_MAXOFREGION,
@@ -49,7 +51,28 @@ enum arithmetic_prog_operators
   ARITHMETIC_OP_COLLAPSE_MIN,
   ARITHMETIC_OP_COLLAPSE_MAX,
   ARITHMETIC_OP_COLLAPSE_MEAN,
+  ARITHMETIC_OP_COLLAPSE_MEDIAN,
   ARITHMETIC_OP_COLLAPSE_NUMBER,
+  ARITHMETIC_OP_COLLAPSE_SIGCLIP_MAD,
+  ARITHMETIC_OP_COLLAPSE_SIGCLIP_STD,
+  ARITHMETIC_OP_COLLAPSE_SIGCLIP_MEAN,
+  ARITHMETIC_OP_COLLAPSE_SIGCLIP_MEDIAN,
+  ARITHMETIC_OP_COLLAPSE_SIGCLIP_NUMBER,
+  ARITHMETIC_OP_COLLAPSE_MADCLIP_MAD,
+  ARITHMETIC_OP_COLLAPSE_MADCLIP_STD,
+  ARITHMETIC_OP_COLLAPSE_MADCLIP_MEAN,
+  ARITHMETIC_OP_COLLAPSE_MADCLIP_MEDIAN,
+  ARITHMETIC_OP_COLLAPSE_MADCLIP_NUMBER,
+  ARITHMETIC_OP_COLLAPSE_SIGCLIP_FILL_MAD,
+  ARITHMETIC_OP_COLLAPSE_SIGCLIP_FILL_STD,
+  ARITHMETIC_OP_COLLAPSE_SIGCLIP_FILL_MEAN,
+  ARITHMETIC_OP_COLLAPSE_SIGCLIP_FILL_MEDIAN,
+  ARITHMETIC_OP_COLLAPSE_SIGCLIP_FILL_NUMBER,
+  ARITHMETIC_OP_COLLAPSE_MADCLIP_FILL_MAD,
+  ARITHMETIC_OP_COLLAPSE_MADCLIP_FILL_STD,
+  ARITHMETIC_OP_COLLAPSE_MADCLIP_FILL_MEAN,
+  ARITHMETIC_OP_COLLAPSE_MADCLIP_FILL_MEDIAN,
+  ARITHMETIC_OP_COLLAPSE_MADCLIP_FILL_NUMBER,
   ARITHMETIC_OP_ADD_DIMENSION_SLOW,
   ARITHMETIC_OP_ADD_DIMENSION_FAST,
   ARITHMETIC_OP_REPEAT,
