@@ -395,10 +395,10 @@ ui_parse_obsline(struct argp_option *option, char *arg,
 /**************************************************************/
 /***************       Sanity Check         *******************/
 /**************************************************************/
-/* Read and check ONLY the options. When arguments are involved, do the
-   check in 'ui_check_options_and_arguments'. */
+/* Check ONLY the options. When arguments are involved, do the check
+   in 'ui_check_options_and_arguments'. */
 static void
-ui_read_check_only_options(struct cosmiccalparams *p)
+ui_check_only_options(struct cosmiccalparams *p)
 {
   int hasobsline=p->obsline!=NULL;
   int hasredshift=!isnan(p->redshift);
@@ -656,9 +656,8 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct cosmiccalparams *p)
   gal_options_read_config_set(&p->cp);
 
 
-  /* Read the options into the program's structure, and check them and
-     their relations prior to printing. */
-  ui_read_check_only_options(p);
+  /* Sanity check only on options. */
+  ui_check_only_options(p);
 
 
   /* Print the option values if asked. Note that this needs to be done
