@@ -5,7 +5,8 @@ Fits is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2022 Free Software Foundation, Inc.
+     Pedram Ashofteh-Ardakani <pedramardakani@pm.me>
+Copyright (C) 2015-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -44,12 +45,12 @@ main (int argc, char *argv[])
   /* Read the input parameters. */
   ui_read_check_inputs_setup(argc, argv, &p);
 
-  /* Run MakeProfiles */
+  /* Run Fits. */
   r=fits(&p);
 
   /* Free all non-freed allocations. */
   ui_free_and_report(&p);
 
-  /* Return successfully.*/
+  /* Return successfully. */
   return r;
 }
