@@ -441,13 +441,13 @@ ui_read_check_mode_keyword(struct fitsparams *p)
 
       /* Give an ID to recognized coordinate systems. */
       if(p->wcscoordsys)
-        p->coordsysid=gal_wcs_coordsys_from_string(p->wcscoordsys);
+        p->coordsysid=gal_wcs_coordsys_name_to_id(p->wcscoordsys);
 
       /* Identify the requested distortion. Note that this also acts as a
          sanity check because it will crash with an error if the given
          string isn't recognized. */
       if(p->wcsdistortion)
-        p->distortionid=gal_wcs_distortion_from_string(p->wcsdistortion);
+        p->distortionid=gal_wcs_distortion_name_to_id(p->wcsdistortion);
 
       /* Make sure the value of '--keyvalue' is actually present. */
       if(p->keyvalue && p->keyvalue->v[0]=='\0')
@@ -530,10 +530,10 @@ ui_read_check_mode_extension(struct fitsparams *p)
 
 
 
-/* Read and check ONLY the options. When arguments are involved, do the
-   check in 'ui_check_options_and_arguments'. */
+/* Check ONLY the options. When arguments are involved, do the check
+   in 'ui_check_options_and_arguments'. */
 static void
-ui_read_check_only_options(struct fitsparams *p)
+ui_check_only_options(struct fitsparams *p)
 {
   /* Check the given optinos and set the operating mode accordingly. */
   ui_read_check_mode_keyword(p);
@@ -819,9 +819,8 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct fitsparams *p)
   gal_options_read_config_set(&p->cp);
 
 
-  /* Read the options into the program's structure, and check them and
-     their relations prior to printing. */
-  ui_read_check_only_options(p);
+  /* Sanity check only on options. */
+  ui_check_only_options(p);
 
 
   /* Print the option values if asked. Note that this needs to be done
