@@ -520,10 +520,10 @@ ui_parse_coordinate_mode(struct argp_option *option, char *arg,
 /**************************************************************/
 /***************       Sanity Check         *******************/
 /**************************************************************/
-/* Read and check ONLY the options. When arguments are involved, do the
+/* Check ONLY the options. When arguments are involved, do the
    check in 'ui_check_options_and_arguments'. */
 static void
-ui_read_check_only_options(struct mkprofparams *p)
+ui_check_only_options(struct mkprofparams *p)
 {
   size_t i;
 
@@ -598,7 +598,7 @@ ui_read_check_only_options(struct mkprofparams *p)
 
 
 /* Sanity check on options AND arguments. If only option values are to be
-   checked, use 'ui_read_check_only_options'. */
+   checked, use 'ui_check_only_options'. */
 static void
 ui_check_options_and_arguments(struct mkprofparams *p)
 {
@@ -2163,9 +2163,8 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct mkprofparams *p)
   gal_options_read_config_set(&p->cp);
 
 
-  /* Read the options into the program's structure, and check them and
-     their relations prior to printing. */
-  ui_read_check_only_options(p);
+  /* Sanity check only on options. */
+  ui_check_only_options(p);
 
 
   /* Print the option values if asked. Note that this needs to be done
