@@ -260,10 +260,10 @@ ui_strlist_to_str(gal_list_str_t *input)
 /**************************************************************/
 /***************       Sanity Check         *******************/
 /**************************************************************/
-/* Read and check ONLY the options. When arguments are involved, do the
-   check in 'ui_check_options_and_arguments'. */
+/* Check ONLY the options. When arguments are involved, do the check
+   in 'ui_check_options_and_arguments'. */
 static void
-ui_read_check_only_options(struct queryparams *p)
+ui_check_only_options(struct queryparams *p)
 {
   size_t i;
   double *darray;
@@ -563,9 +563,8 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct queryparams *p)
   gal_options_read_config_set(&p->cp);
 
 
-  /* Read the options into the program's structure, and check them and
-     their relations prior to printing. */
-  ui_read_check_only_options(p);
+  /* Sanity check only on options. */
+  ui_check_only_options(p);
 
 
   /* Print the option values if asked. Note that this needs to be done
@@ -616,5 +615,7 @@ ui_free_report(struct queryparams *p, struct timeval *t1)
   /* Free the allocated arrays: */
   free(p->cp.hdu);
   free(p->cp.output);
+  free(p->datasetstr);
+  free(p->datasetuse);
   free(p->downloadname);
 }
