@@ -385,10 +385,10 @@ ui_read_quantile_range(struct argp_option *option, char *arg,
 /**************************************************************/
 /***************       Sanity Check         *******************/
 /**************************************************************/
-/* Read and check ONLY the options. When arguments are involved, do the
-   check in 'ui_check_options_and_arguments'. */
+/* Check ONLY the options. When arguments are involved, do the check
+   in 'ui_check_options_and_arguments'. */
 static void
-ui_read_check_only_options(struct statisticsparams *p)
+ui_check_only_options(struct statisticsparams *p)
 {
   gal_list_i32_t *tmp;
   struct gal_tile_two_layer_params *tl=&p->cp.tl;
@@ -409,13 +409,14 @@ ui_read_check_only_options(struct statisticsparams *p)
     {
       /* The tile or sky mode cannot be called with any other modes. */
       if( p->asciihist || p->asciicfp || p->histogram || p->histogram2d
-          || p->cumulative || p->sigmaclip || p->fitname
+          || p->cumulative || p->sigmaclip || p->madclip || p->fitname
           || !isnan(p->mirror) )
         error(EXIT_FAILURE, 0, "'--ontile' or '--sky' cannot be called "
               "with any of the 'particular' calculation options, for "
-              "example '--histogram'. This is because the latter work "
-              "over the whole dataset and element positions are changed, "
-              "but in the former positions are significant");
+              "example '--histogram' or '--madclip'. Because these "
+              "operators change the order of the data and element "
+              "positions are changed, but in the former positions are "
+              "significant");
 
       /* Make sure the tessellation defining options are given. */
       if( tl->tilesize==NULL || tl->numchannels==NULL
@@ -469,12 +470,19 @@ ui_read_check_only_options(struct statisticsparams *p)
     }
 
 
-  /* Sigma-clipping needs 'sclipparams'. */
+  /* Sigma-clipping needs 'sclipparams' and MAD-clipping needs
+     'mclipparams' */
   if(p->sigmaclip && isnan(p->sclipparams[0]))
     error(EXIT_FAILURE, 0, "'--sclipparams' is necessary with "
           "'--sigmaclip'. '--sclipparams' takes two values (separated "
           "by a comma) for defining the sigma-clip: the multiple of "
           "sigma, and tolerance (<1) or number of clips (>1).");
+  if(p->madclip && isnan(p->mclipparams[0]))
+    error(EXIT_FAILURE, 0, "'--mclipparams' is necessary with "
+          "'--madclip' (median absolute deviation clipping). "
+          "'--mclipparams' takes two values (separated "
+          "by a comma) for defining the MAD-clip: the multiple of "
+          "MAD, and tolerance (<1) or number of clips (>1).");
 
 
   /* If any of the mode measurements are requested, then 'mirrordist' is
@@ -491,6 +499,7 @@ ui_read_check_only_options(struct statisticsparams *p)
                 "mode-related single measurements ('--mode', "
                 "'--modequant', '--modesym', and '--modesymvalue')");
         break;
+      case UI_KEY_SIGCLIPMAD:
       case UI_KEY_SIGCLIPSTD:
       case UI_KEY_SIGCLIPMEAN:
       case UI_KEY_SIGCLIPNUMBER:
@@ -800,17 +809,23 @@ ui_make_sorted_if_necessary(struct statisticsparams *p)
       case UI_KEY_MEDIAN:
       case UI_KEY_QUANTILE:
       case UI_KEY_QUANTFUNC:
+      case UI_KEY_MADCLIPMAD:
+      case UI_KEY_SIGCLIPMAD:
+      case UI_KEY_MADCLIPSTD:
       case UI_KEY_SIGCLIPSTD:
       case UI_KEY_QUANTOFMEAN:
       case UI_KEY_SIGCLIPMEAN:
+      case UI_KEY_MADCLIPMEAN:
+      case UI_KEY_MADCLIPNUMBER:
       case UI_KEY_SIGCLIPNUMBER:
+      case UI_KEY_MADCLIPMEDIAN:
       case UI_KEY_SIGCLIPMEDIAN:
         is_necessary=1;
         break;
       }
 
   /* Check in the rest of the outputs. */
-  if( is_necessary==0 && ( p->sigmaclip || !isnan(p->mirror) ) )
+  if( p->sigmaclip || p->madclip || !isnan(p->mirror) )
     is_necessary=1;
 
   /* Do the sorting, we will keep the sorted array in a separate space,
@@ -1016,7 +1031,7 @@ ui_preparations_fitestimate(struct statisticsparams *p)
         }
 
       /* Make sure a HDU is specified. We need to do this here (not
-         in 'ui_read_check_only_options') because only here we know
+         in 'ui_check_only_options') because only here we know
          that the user specified a file not a value. */
       if( gal_fits_name_is_fits(p->fitestimate) && p->fitestimatehdu==NULL )
         error(EXIT_FAILURE, 0, "no HDU specified for '%s' (given to "
@@ -1024,7 +1039,7 @@ ui_preparations_fitestimate(struct statisticsparams *p)
               "option to specify the HDU", p->fitestimate);
 
       /* Make sure a column is specified. We need to do this here (not
-         in 'ui_read_check_only_options') because only here we know
+         in 'ui_check_only_options') because only here we know
          that the user specified a file not a value. */
       if( p->fitestimatecol==NULL )
         error(EXIT_FAILURE, 0, "no column specified for '%s' (given to "
@@ -1300,9 +1315,8 @@ ui_read_check_inputs_setup(int argc, char *argv[],
   gal_options_read_config_set(&p->cp);
 
 
-  /* Read the options into the program's structure, and check them and
-     their relations prior to printing. */
-  ui_read_check_only_options(p);
+  /* Sanity check only on options. */
+  ui_check_only_options(p);
 
 
   /* Print the option values if asked. Note that this needs to be done
