@@ -5,7 +5,7 @@ Table is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2019-2022 Free Software Foundation, Inc.
+Copyright (C) 2019-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -30,11 +30,15 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 #include <gnuastro/wcs.h>
 #include <gnuastro/type.h>
+#include <gnuastro/pointer.h>
+#include <gnuastro/statistics.h>
 
 #include <gnuastro-internal/checkset.h>
 #include <gnuastro-internal/arithmetic-set.h>
 
 #include "main.h"
+
+#include "ui.h"
 #include "arithmetic.h"
 
 
@@ -136,8 +140,16 @@ arithmetic_operator_name(int operator)
       case ARITHMETIC_TABLE_OP_IMGTOWCS: out="img-to-wcs"; break;
       case ARITHMETIC_TABLE_OP_DATETOSEC: out="date-to-sec"; break;
       case ARITHMETIC_TABLE_OP_DISTANCEFLAT: out="distance-flat"; break;
-      case ARITHMETIC_TABLE_OP_DATETOMILLISEC: out="date-to-millisec"; break;
-      case ARITHMETIC_TABLE_OP_DISTANCEONSPHERE: out="distance-on-sphere"; break;
+      case ARITHMETIC_TABLE_OP_DATETOMILLISEC:
+        out="date-to-millisec"; break;
+      case ARITHMETIC_TABLE_OP_DISTANCEONSPHERE:
+        out="distance-on-sphere"; break;
+      case ARITHMETIC_TABLE_OP_SORTEDTOINTERVAL:
+        out="sorted-to-interval"; break;
+      case ARITHMETIC_TABLE_OP_EQJ2000TOFLAT:
+        out="eq-j2000-to-flat"; break;
+      case ARITHMETIC_TABLE_OP_EQJ2000FROMFLAT:
+        out="eq-j2000-from-flat"; break;
       default:
         error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix "
               "the problem. %d is not a recognized operator code", __func__,
@@ -164,7 +176,7 @@ arithmetic_init_wcs(struct tableparams *p, char *operator)
 
       /* Read the WCS. */
       p->wcs=gal_wcs_read(p->wcsfile, p->wcshdu, p->cp.wcslinearmatrix,
-                          0, 0, &p->nwcs);
+                          0, 0, &p->nwcs, "--wcshdu");
       if(p->wcs==NULL)
         error(EXIT_FAILURE, 0, "%s (hdu: %s): no WCS could be read by "
               "WCSLIB", p->wcsfile, p->wcshdu);
@@ -194,6 +206,10 @@ arithmetic_set_operator(struct tableparams *p, char *string,
         { op=ARITHMETIC_TABLE_OP_WCSTOIMG; *num_operands=0; }
       else if( !strcmp(string, "img-to-wcs"))
         { op=ARITHMETIC_TABLE_OP_IMGTOWCS; *num_operands=0; }
+      else if( !strcmp(string, "eq-j2000-to-flat"))
+        { op=ARITHMETIC_TABLE_OP_EQJ2000TOFLAT; *num_operands=0; }
+      else if( !strcmp(string, "eq-j2000-from-flat"))
+        { op=ARITHMETIC_TABLE_OP_EQJ2000FROMFLAT; *num_operands=0; }
       else if( !strcmp(string, "date-to-sec"))
         { op=ARITHMETIC_TABLE_OP_DATETOSEC; *num_operands=0; }
       else if( !strcmp(string, "date-to-millisec"))
@@ -202,6 +218,8 @@ arithmetic_set_operator(struct tableparams *p, char *string,
         { op=ARITHMETIC_TABLE_OP_DISTANCEFLAT; *num_operands=0; }
       else if( !strcmp(string, "distance-on-sphere"))
         { op=ARITHMETIC_TABLE_OP_DISTANCEONSPHERE; *num_operands=0; }
+      else if( !strcmp(string, "sorted-to-interval"))
+        { op=ARITHMETIC_TABLE_OP_SORTEDTOINTERVAL; *num_operands=0; }
       else
         { op=GAL_ARITHMETIC_OP_INVALID; *num_operands=GAL_BLANK_INT; }
     }
@@ -370,7 +388,7 @@ arithmetic_indexs_final(struct tableparams *p)
   p->colarray=gal_list_data_to_array_ptr(p->table, &p->numcolarray);
 
   /* go over each package of columns. */
-  for(tmp=p->outcols;tmp!=NULL;tmp=tmp->next)
+  for(tmp=p->colpack;tmp!=NULL;tmp=tmp->next)
     {
       /* If we are on an arithmetic operation. */
       if(tmp->arith)
@@ -434,20 +452,64 @@ arithmetic_indexs_final(struct tableparams *p)
 /*********************************************************************/
 /********************       Low-level tools      *********************/
 /*********************************************************************/
+static void
+arithmetic_placeholder_name(gal_data_t *col)
+{
+  static size_t counter=0;
+
+  /* Increment counter next time this function is called. */
+  ++counter;
+
+  /* Free any possibly existing metadata. */
+  if(col->name)    free(col->name);
+  if(col->unit)    free(col->unit);
+  if(col->comment) free(col->comment);
+
+  /* Set the new meta-data. */
+  errno=0;
+  if( asprintf(&col->name, "ARITH_%zu", counter)==-1 )
+    error(EXIT_FAILURE, errno, "%s: asprintf error for name", __func__);
+  if( asprintf(&col->unit, "arith_unit_%zu", counter)==-1)
+    error(EXIT_FAILURE, errno, "%s: asprintf error for unit", __func__);
+  if( asprintf(&col->comment, "Column from arithmetic operation %zu",
+               counter)==-1 )
+    error(EXIT_FAILURE, errno, "%s: asprintf error for comment", __func__);
+}
+
+
+
+
+
 static gal_data_t *
 arithmetic_stack_pop(gal_data_t **stack, int operator, char *errormsg)
 {
-  gal_data_t *out=*stack;
+  size_t i;
+  gal_data_t *out, *top=*stack;
 
-  /* Update the stack. */
-  if(*stack)
-    *stack=(*stack)->next;
-  else
-    error(EXIT_FAILURE, 0, "not enough operands for '%s'%s",
-          arithmetic_operator_name(operator), errormsg?errormsg:"");
+  /* Update the stack (where necessary). */
+  switch(operator)
+    {
+    /* Operators don't need to pop the dataset from the stack (they just
+       need the metadata). */
+    case GAL_ARITHMETIC_OP_INDEX:
+    case GAL_ARITHMETIC_OP_COUNTER:
+      out=gal_data_alloc_empty(top->ndim, top->minmapsize, top->quietmmap);
+      for(i=0;i<top->ndim;++i) out->dsize[i]=top->dsize[i];
+      out->size=top->size;
+      break;
+
+    /* Operators that actually need the data. */
+    default:
+      out=*stack;
+      if(*stack)
+        *stack=(*stack)->next;
+      else
+        error(EXIT_FAILURE, 0, "not enough operands for '%s'%s",
+              arithmetic_operator_name(operator), errormsg?errormsg:"");
+    }
 
   /* Arithmetic changes the contents of a dataset, so the old name (in the
-     FITS 'EXTNAME' keyword) must not be used beyond this
+     FITS 'EXTNAME' keyword) or units must not be used beyond this
      point. Furthermore, in Arithmetic, the 'name' element is used to
      identify variables (with the 'set-' operator). */
   if(out->name)    { free(out->name);    out->name=NULL;    }
@@ -583,12 +645,16 @@ arithmetic_wcs(struct tableparams *p, gal_data_t **stack, int operator)
 
       /* For image coordinates, we don't need much precision. */
       for(i=0;i<ndim;++i)
-        coord[i]=gal_data_copy_to_new_type_free(coord[i], GAL_TYPE_FLOAT32);
+        coord[i]=gal_data_copy_to_new_type_free(coord[i],
+                                                GAL_TYPE_FLOAT32);
 
       /* Set the names, units and comments for each dataset. */
-      arithmetic_update_metadata(coord[0],"X","pixel","Converted from WCS");
-      arithmetic_update_metadata(coord[1],"Y","pixel","Converted from WCS");
-      arithmetic_update_metadata(coord[2],"Z","pixel","Converted from WCS");
+      arithmetic_update_metadata(coord[0], "X", "pixel",
+                                 "Converted from WCS");
+      arithmetic_update_metadata(coord[1], "Y", "pixel",
+                                 "Converted from WCS");
+      arithmetic_update_metadata(coord[2], "Z", "pixel",
+                                 "Converted from WCS");
     }
   else
     {
@@ -615,6 +681,107 @@ arithmetic_wcs(struct tableparams *p, gal_data_t **stack, int operator)
 
 
 
+static void
+arithmetic_curved_flat(struct tableparams *p, gal_data_t **stack,
+                       int operator)
+{
+  struct wcsprm *wcs=NULL;
+  char *ctype[2], *cunit[2];
+  gal_data_t *w1, *w2, *proj, *refw1, *refw2;
+  double ref[2], cdelt[2]={1.0f,1.0f}, pc[4]={-1,0,0,1};
+  char **strarr, ctype1[9], ctype2[9]; /* 9=8+1 (keylength + '\0') */
+
+  /* Pop the necessary datasets from the stack (order is important). */
+  proj=arithmetic_stack_pop(stack, operator, NULL); /* 1st: projection.*/
+  refw2=arithmetic_stack_pop(stack, operator, NULL);/* 2nd: Ref. Dec. */
+  refw1=arithmetic_stack_pop(stack, operator, NULL);/* 3rd: Ref. RA. */
+  w2=arithmetic_stack_pop(stack, operator, NULL);   /* 4th: RA column. */
+  w1=arithmetic_stack_pop(stack, operator, NULL);   /* 5th: Dec column. */
+
+  /* Basic sanity checks. */
+  strarr=proj->array;
+  if(proj->size!=1 || refw1->size!=1 || refw2->size!=1)
+    error(EXIT_FAILURE, 0, "the first, second and third popped operands "
+          "from the '%s' operator should have a single element (they "
+          "should not be columns, but a single value), but they "
+          "respectively have %zu, %zu, %zu columns. Recall that the "
+          "first popped operand is the right-most operand (closest to "
+          "the operator", arithmetic_operator_name(operator),
+          proj->size, refw1->size, refw2->size);
+  if(proj->type!=GAL_TYPE_STRING)
+    error(EXIT_FAILURE, 0, "the first popped operand to the '%s' "
+          "operator must be a string, not numeric. It should be the "
+          "three character projection identifier of the WCS standard "
+          "in FITS (for example 'TAN' or 'MOL')",
+          arithmetic_operator_name(operator));
+  if(gal_wcs_projection_name_to_id(strarr[0])==GAL_WCS_PROJECTION_INVALID)
+    error(EXIT_FAILURE, 0, "the first popped operand to the '%s' "
+          "operator ('%s') could not be interpretted as a pre-defined "
+          "projection in the WCS stanadard of FITS",
+          arithmetic_operator_name(operator), strarr[0]);
+
+  /* Convert the numeric inputs into the float64 and put the reference
+     values into the expected array.*/
+  refw1=gal_data_copy_to_new_type_free(refw1, GAL_TYPE_FLOAT64);
+  refw2=gal_data_copy_to_new_type_free(refw2, GAL_TYPE_FLOAT64);
+  w1=gal_data_copy_to_new_type_free(w1, GAL_TYPE_FLOAT64);
+  w2=gal_data_copy_to_new_type_free(w2, GAL_TYPE_FLOAT64);
+  ref[0]=((double *)(refw1->array))[0];
+  ref[1]=((double *)(refw2->array))[0];
+
+  /* Construct the CTYPEi strings */
+  sprintf(ctype1, "%s---%s", "RA", strarr[0]);
+  sprintf(ctype2, "%s--%s", "DEC", strarr[0]);
+  ctype[0]=ctype1;
+  ctype[1]=ctype2;
+  cunit[0]="deg";
+  cunit[1]="deg";
+
+  /* Build the WCS structure for the conversion. */
+  wcs=gal_wcs_create(ref, ref, cdelt, pc, cunit, ctype, 2,
+                     GAL_WCS_LINEAR_MATRIX_PC);
+
+  /* For a check:
+  int nkeyrec;
+  char *wcsstr;
+  wcsstr=gal_wcs_write_wcsstr(wcs, &nkeyrec);
+  printf("%s\n", wcsstr); exit(0);
+  */
+
+  /* Put the second world coordinate as the next token of the first, and do
+     the conversion. */
+  w1->next=w2;
+  switch(operator)
+    {
+    case ARITHMETIC_TABLE_OP_EQJ2000TOFLAT:
+      gal_wcs_world_to_img(w1, wcs, 1);
+      break;
+    case ARITHMETIC_TABLE_OP_EQJ2000FROMFLAT:
+      gal_wcs_img_to_world(w1, wcs, 1);
+      break;
+    default:
+      error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at '%s' to "
+            "fix the problem. The integer '%d' is not a recognized "
+            "operator identifier", __func__, PACKAGE_BUGREPORT, operator);
+    }
+
+  /* Put the output datasets in the stack in reverse order. */
+  w1->next=w2->next=NULL;
+  gal_list_data_add(stack, w1);
+  gal_list_data_add(stack, w2);
+
+  /* Clean up (the coordinate conversion was done in place, so 'w1' and
+     'w2' are the output datasets). */
+  gal_wcs_free(wcs);
+  gal_data_free(proj);
+  gal_data_free(refw1);
+  gal_data_free(refw2);
+}
+
+
+
+
+
 static double
 arithmetic_distance_flat(double a1, double a2, double b1, double b2)
 {
@@ -627,7 +794,8 @@ arithmetic_distance_flat(double a1, double a2, double b1, double b2)
 
 
 static void
-arithmetic_distance(struct tableparams *p, gal_data_t **stack, int operator)
+arithmetic_distance(struct tableparams *p, gal_data_t **stack,
+                    int operator)
 {
   size_t i, j;
   double *o, *a1, *a2, *b1, *b2;
@@ -696,9 +864,10 @@ arithmetic_distance(struct tableparams *p, gal_data_t **stack, int operator)
       if(a->size==1 || b->size==1) /* One of them is a single point. */
         for(i=0;i<a->size;++i)
           for(j=0;j<b->size;++j)
-            o[a->size>b->size?i:j] = distance_func(a1[i], a2[i], b1[j], b2[j]);
+            o[a->size>b->size?i:j] = distance_func(a1[i], a2[i], b1[j],
+                                                   b2[j]);
       else                     /* Both have the same length. */
-        for(i=0;i<a->size;++i) /* (all were originally from the same table) */
+        for(i=0;i<a->size;++i) /* (all were originally from same table) */
           o[i] = distance_func(a1[i], a2[i], b1[i], b2[i]);
     }
 
@@ -789,6 +958,75 @@ arithmetic_datetosec(struct tableparams *p, gal_data_t **stack,
 
 
 
+/* Convert the ISO date format to seconds since Unix time. */
+static void
+arithmetic_sortedtointerval(struct tableparams *p, gal_data_t **stack,
+                            int operator)
+{
+  int f32out;
+  size_t i, size;
+  double *in, *min, *max;
+  gal_data_t *min_d, *max_d;
+
+  /* Input dataset. */
+  gal_data_t *in_d=arithmetic_stack_pop(stack, operator, NULL);
+
+  /* Make sure the input is sorted and is numeric */
+  if(in_d->type==GAL_TYPE_STRING)
+    error(EXIT_FAILURE, 0, "the operand given to 'sortedtointerval' "
+          "should have a numeric data type");
+  if(gal_statistics_is_sorted(in_d, 1)==0)
+    error(EXIT_FAILURE, 0, "the operand given to 'sortedtointerval' "
+          "should be sorted");
+
+  /* Convert the input to double precision for internal processing. In the
+     end we'll convert the outputs to float32 unless the input was
+     originally float64. */
+  f32out = in_d->type!=GAL_TYPE_FLOAT64;
+  in_d=gal_data_copy_to_new_type_free(in_d, GAL_TYPE_FLOAT64);
+
+  /* Allocate the output columns (minimum and maximum of each interval). */
+  min_d=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, in_d->dsize, NULL, 0,
+                       in_d->minmapsize, in_d->quietmmap,
+                       NULL, NULL, NULL);
+  max_d=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, in_d->dsize, NULL, 0,
+                       in_d->minmapsize, in_d->quietmmap,
+                       NULL, NULL, NULL);
+
+  /* Fill the minimum and maximum intervals. Just note that the minimum
+     value of the first interval and maximum of the last element need
+     special attention (we'll extrapolate from the after/before elements to
+     preserve constant spacing in case that is the case). */
+  in=in_d->array;
+  size=in_d->size;
+  min=min_d->array;
+  max=max_d->array;
+  min[0]      = in[0]      - (in[1]      - in[0]     )/2;
+  max[0]      = in[0]      + (in[1]      - in[0]     )/2;
+  min[size-1] = in[size-1] - (in[size-1] - in[size-2])/2;
+  max[size-1] = in[size-1] + (in[size-1] - in[size-2])/2;
+  for(i=1;i<size-1;++i)
+    {
+      min[i] = max[i-1];
+      max[i] = in[i] + (in[i+1]-in[i])/2;
+    }
+
+  /* Convert the output to float32, unless the input was float64. */
+  if(f32out)
+    {
+      min_d=gal_data_copy_to_new_type_free(min_d, GAL_TYPE_FLOAT32);
+      max_d=gal_data_copy_to_new_type_free(max_d, GAL_TYPE_FLOAT32);
+    }
+
+  /* Clean up and put the resulting calculation back on the stack. */
+  max_d->next=min_d;
+  gal_data_free(in_d);
+  gal_list_data_add(stack, max_d);
+}
+
+
+
+
 
 
 
@@ -807,34 +1045,6 @@ arithmetic_datetosec(struct tableparams *p, gal_data_t **stack,
 /*********************************************************************/
 /********************          Operations        *********************/
 /*********************************************************************/
-static void
-arithmetic_placeholder_name(gal_data_t *col)
-{
-  static size_t counter=0;
-
-  /* Increment counter next time this function is called. */
-  ++counter;
-
-  /* Free any possibly existing metadata. */
-  if(col->name)    free(col->name);
-  if(col->unit)    free(col->unit);
-  if(col->comment) free(col->comment);
-
-  /* Set the new meta-data. */
-  errno=0;
-  if( asprintf(&col->name, "ARITH_%zu", counter)==-1 )
-    error(EXIT_FAILURE, errno, "%s: asprintf error for name", __func__);
-  if( asprintf(&col->unit, "arith_unit_%zu", counter)==-1)
-    error(EXIT_FAILURE, errno, "%s: asprintf error for unit", __func__);
-  if( asprintf(&col->comment, "Column from arithmetic operation %zu",
-               counter)==-1 )
-    error(EXIT_FAILURE, errno, "%s: asprintf error for comment", __func__);
-}
-
-
-
-
-
 static void
 arithmetic_operator_run(struct tableparams *p,
                         struct arithmetic_token *token,
@@ -842,7 +1052,7 @@ arithmetic_operator_run(struct tableparams *p,
                         gal_data_t **stack)
 {
   int flags=GAL_ARITHMETIC_FLAGS_BASIC;
-  gal_data_t *d1=NULL, *d2=NULL, *d3=NULL;
+  gal_data_t *d1=NULL, *d2=NULL, *d3=NULL, *d4=NULL, *d5=NULL;
 
   /* Set the operating-mode flags if necessary. */
   if(p->cp.quiet) flags |= GAL_ARITHMETIC_FLAG_QUIET;
@@ -876,6 +1086,21 @@ arithmetic_operator_run(struct tableparams *p,
           d1=arithmetic_stack_pop(stack, token->operator, NULL);
           break;
 
+        case 4:
+          d4=arithmetic_stack_pop(stack, token->operator, NULL);
+          d3=arithmetic_stack_pop(stack, token->operator, NULL);
+          d2=arithmetic_stack_pop(stack, token->operator, NULL);
+          d1=arithmetic_stack_pop(stack, token->operator, NULL);
+          break;
+
+        case 5:
+          d5=arithmetic_stack_pop(stack, token->operator, NULL);
+          d4=arithmetic_stack_pop(stack, token->operator, NULL);
+          d3=arithmetic_stack_pop(stack, token->operator, NULL);
+          d2=arithmetic_stack_pop(stack, token->operator, NULL);
+          d1=arithmetic_stack_pop(stack, token->operator, NULL);
+          break;
+
         case -1:
           error(EXIT_FAILURE, 0, "operators with a variable number of "
                 "operands are not yet implemented. Please contact us at "
@@ -897,7 +1122,7 @@ arithmetic_operator_run(struct tableparams *p,
          ignored. */
       gal_list_data_add(stack, gal_arithmetic(token->operator,
                                               p->cp.numthreads,
-                                              flags, d1, d2, d3) );
+                                              flags, d1, d2, d3, d4, d5) );
 
       /* Reset the meta-data for the element that was just put on the
          stack. */
@@ -914,6 +1139,11 @@ arithmetic_operator_run(struct tableparams *p,
           arithmetic_wcs(p, stack, token->operator);
           break;
 
+        case ARITHMETIC_TABLE_OP_EQJ2000TOFLAT:
+        case ARITHMETIC_TABLE_OP_EQJ2000FROMFLAT:
+          arithmetic_curved_flat(p, stack, token->operator);
+          break;
+
         case ARITHMETIC_TABLE_OP_DATETOSEC:
         case ARITHMETIC_TABLE_OP_DATETOMILLISEC:
           arithmetic_datetosec(p, stack, token->operator);
@@ -924,6 +1154,10 @@ arithmetic_operator_run(struct tableparams *p,
           arithmetic_distance(p, stack, token->operator);
           break;
 
+        case ARITHMETIC_TABLE_OP_SORTEDTOINTERVAL:
+          arithmetic_sortedtointerval(p, stack, token->operator);
+          break;
+
         case ARITHMETIC_TABLE_OP_SET:
           gal_arithmetic_set_name(setprm, token->name_def);
           break;
@@ -947,9 +1181,11 @@ gal_data_t *
 arithmetic_read_at_usage(struct tableparams *p,
                          struct arithmetic_token *token)
 {
-  char *c;
-  size_t i;
+  uint8_t pid;
+  size_t i, one=1;
+  gal_data_t *tmp;
   int hasnewline=0;
+  char *c, **strarr;
 
   /* If the number is blank, then we have a name and should return the
      first column in the table that has the given name. */
@@ -961,6 +1197,27 @@ arithmetic_read_at_usage(struct tableparams *p,
             && strcasecmp(p->colarray[i]->name, token->id_at_usage)==0 )
           return p->colarray[i];
 
+      /* If control reaches here, then the requested name didn't exist in
+         the table, if the next token is the equatorial on flat operator,
+         then we should also check known projection identifiers. */
+      if( token->next
+          && ( token->next->operator==ARITHMETIC_TABLE_OP_EQJ2000TOFLAT
+            || token->next->operator==ARITHMETIC_TABLE_OP_EQJ2000FROMFLAT) )
+        {
+          /* If the projection is recognized, add it to the stack
+             otherwise, just let the default error finish the program. */
+          pid=gal_wcs_projection_name_to_id(token->id_at_usage);
+          if(pid!=GAL_WCS_PROJECTION_INVALID)
+            {
+              tmp=gal_data_alloc(NULL, GAL_TYPE_STRING, 1, &one, NULL, 0,
+                                 p->cp.minmapsize, p->cp.quietmmap, NULL,
+                                 NULL, NULL);
+              strarr=tmp->array;
+              gal_checkset_allocate_copy(token->id_at_usage, &strarr[0]);
+              return tmp;
+            }
+        }
+
       /* If control reaches here, then the requested name didn't exist in
          the table. This may happen because the user broke an arithmetic
          command into muliple lines and forgot to put a back-slash. If so,
@@ -1036,7 +1293,7 @@ arithmetic_reverse_polish(struct tableparams *p,
           token->loadcol=NULL;
         }
 
-      /* Constant number: just put it ontop of the stack. */
+      /* Constant number: just put it on top of the stack. */
       else if(token->constant)
         {
           gal_list_data_add(&stack, token->constant);
@@ -1054,7 +1311,12 @@ arithmetic_reverse_polish(struct tableparams *p,
 
       /* A column from the table. */
       else if(token->index!=GAL_BLANK_SIZE_T)
-        gal_list_data_add(&stack, p->colarray[token->index]);
+        {
+          if(p->colarray[token->index]->ndim!=1)
+            error(EXIT_FAILURE, 0, "column arithmetic currently only works "
+                  "on single-valued columns, not vector columns");
+          gal_list_data_add(&stack, p->colarray[token->index]);
+        }
 
       /* Un-recognized situation. */
       else
@@ -1066,10 +1328,10 @@ arithmetic_reverse_polish(struct tableparams *p,
       ++setprm.tokencounter;
     }
 
-  /* Put everything that remains in the stack (reversed) into the final
-     table. Just note that 'gal_list_data_add' behaves differently for
-     lists, so we'll add have to manually set the 'next' element to NULL
-     before adding the column to the final table. */
+ /* Put everything that remains in the stack (reversed) into the final
+    table. Just note that 'gal_list_data_add' behaves differently for
+    lists, so we'll add have to manually set the 'next' element to NULL
+    before adding the column to the final table. */
   gal_list_data_reverse(&stack);
   while(stack!=NULL)
     {
@@ -1134,7 +1396,7 @@ arithmetic_operate(struct tableparams *p)
   p->table=NULL;
 
   /* Go over each package of columns. */
-  for(outpack=p->outcols; outpack!=NULL; outpack=outpack->next)
+  for(outpack=p->colpack; outpack!=NULL; outpack=outpack->next)
     {
       if(outpack->arith)
         arithmetic_reverse_polish(p, outpack);
@@ -1149,4 +1411,10 @@ arithmetic_operate(struct tableparams *p)
      column contents have either been moved into the new table, or have
      already been freed. */
   gal_list_data_reverse(&p->table);
+
+  /* Clean up. */
+  ui_colpack_free(p->colpack);
+  if(p->colarray) free(p->colarray);
+  p->colarray=NULL;
+  p->colpack=NULL;
 }
