@@ -5,7 +5,7 @@ Table is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2022 Free Software Foundation, Inc.
+Copyright (C) 2016-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -66,14 +66,15 @@ static char
 args_doc[] = "ASTRdata";
 
 const char
-doc[] = GAL_STRINGS_TOP_HELP_INFO PROGRAM_NAME" can be used to view the "
-  "information, select columns, or convert tables. The inputs and outputs "
-  "can be plain text (with white-space or comma as delimiters), FITS ascii, "
-  "or FITS binary tables. The output columns can either be selected by "
-  "number (counting from 1), name or using regular expressions. For regular "
-  "expressions, enclose the value to the '--column' ('-c') option in "
-  "slashes ('\\', as in '-c\\^mag\\'). To print the selected columns on the "
-  "command-line, don't specify an output file.\n"
+doc[] = GAL_STRINGS_TOP_HELP_INFO PROGRAM_NAME" can be used to view "
+  "the information, select columns, or convert tables. The inputs and "
+  "outputs can be plain text (with white-space or comma as delimiters), "
+  "FITS ascii, or FITS binary tables. The output columns can either be "
+  "selected by number (counting from 1), name or using regular "
+  "expressions. For regular expressions, enclose the value to the "
+  "'--column' ('-c') option in slashes ('\\', as in '-c\\^mag\\'). "
+  "To print the selected columns on the command-line, don't specify "
+  "an output file.\n"
   GAL_STRINGS_MORE_HELP_INFO
   /* After the list of options: */
   "\v"
@@ -120,6 +121,8 @@ ui_initialize_options(struct tableparams *p,
   cp->coptions           = gal_commonopts_options;
 
   /* Program-specific initialization. */
+  p->txtf32precision     = GAL_BLANK_INT;
+  p->txtf64precision     = GAL_BLANK_INT;
   p->head                = GAL_BLANK_SIZE_T;
   p->tail                = GAL_BLANK_SIZE_T;
 
@@ -187,7 +190,8 @@ parse_opt(int key, char *arg, struct argp_state *state)
          'arg' will be an empty string! We don't want to account for such
          cases (and give a clear error that no input has been given). */
       if(p->filename)
-        argp_error(state, "only one argument (input file) should be given");
+        argp_error(state, "only one argument (input file) should be given, "
+                   "extra argument is: '%s'", arg);
       else
         if(arg[0]!='\0') p->filename=arg;
       break;
@@ -222,11 +226,12 @@ parse_opt(int key, char *arg, struct argp_state *state)
 /**************************************************************/
 /***************       Sanity Check         *******************/
 /**************************************************************/
-/* Read and check ONLY the options. When arguments are involved, do the
-   check in 'ui_check_options_and_arguments'. */
+/* Check ONLY the options. When arguments are involved, do the check
+   in 'ui_check_options_and_arguments'. */
 static void
-ui_read_check_only_options(struct tableparams *p)
+ui_check_only_options(struct tableparams *p)
 {
+  size_t i;
   double *darr;
   gal_data_t *tmp;
 
@@ -274,15 +279,55 @@ ui_read_check_only_options(struct tableparams *p)
     }
 
   /* Make sure only one of the positional row selection operations is
-     called in this run.*/
-  if(p->rowlimit
-     && p->rowrandom
-     && p->head!=GAL_BLANK_SIZE_T
-     && p->tail!=GAL_BLANK_SIZE_T)
+     called in one run. */
+  if( (p->rowrange!=NULL)
+      + (p->rowrandom!=0)
+      + (p->head!=GAL_BLANK_SIZE_T)
+      + (p->tail!=GAL_BLANK_SIZE_T) > 1 )
     error(EXIT_FAILURE, 0, "only one of the following options can be "
-          "called in one run: '--head', '--tail', '--rowlimit' and "
+          "called in one run: '--head', '--tail', '--rowrange' and "
           "'--rowrandom'");
 
+  /* Make sure the value to '--rowrange' is in the correct format. */
+  if(p->rowrange)
+    {
+      /* There should only be two values. */
+      if(p->rowrange->size!=2)
+        error(EXIT_FAILURE, 0, "only two should be given to "
+              "'--rowrange' (the top and bottom row numbers specifying "
+              "your desired range)");
+
+      /* Do individual checks. */
+      darr=p->rowrange->array;
+      for(i=0;i<p->rowrange->size;++i)
+        {
+          /* Make sure it isn't 0 or negative. */
+          if( darr[i]<=0 )
+            error(EXIT_FAILURE, 0, "%g (value given to '--rowrange') "
+                  "is smaller than, or equal to, zero! This option's "
+                  "values are row-counters (starting from 1), so they "
+                  "must be positive integers", darr[i]);
+
+          /* Make sure its an integer. */
+          if( darr[i] != (size_t)(darr[i]) )
+            error(EXIT_FAILURE, 0, "%g (value given to '--rowrange') is "
+                  "not an integer! This option's values are row-counters "
+                  "so they must be integers.", darr[i]);
+
+          /* Subtract 1 from the value, so it counts from 0. */
+          --darr[i];
+        }
+
+      /* Make sure that the first value is smaller than the second. */
+      if( darr[0] > darr[1] )
+        error(EXIT_FAILURE, 0, "the first value to '--rowrange' (%g) is "
+              "larger than the second (%g). This option's values defines "
+              "a row-counter interval, assuming the first value is the top "
+              "of the desired interval (smaller row counter) and the "
+              "second value is the bottom of the desired interval (larger "
+              "row counter)", darr[0], darr[1]);
+    }
+
   /* If '--colmetadata' is given, make sure none of the given options have
      more than three values. */
   if(p->colmetadata)
@@ -292,10 +337,43 @@ ui_read_check_only_options(struct tableparams *p)
               "call of '--colmetadata' ('-m') after the original columns "
               "name or number. But %zu strings have been given", tmp->size);
 
-  /* '--head' and '--tail' shouldn't be called together! */
-  if(p->head!=GAL_BLANK_SIZE_T && p->tail!=GAL_BLANK_SIZE_T)
-    error(EXIT_FAILURE, 0, "'--head' and '--tail' can't be called at "
-          "the same time");
+  /* If '--catcolumns' is given (to only concatenate certain columns), but
+     no file has been given for appending from (to '--catcolumnfile'), then
+     print a warning to let the user know (this may have been a typo!). */
+  if(p->cp.quiet==0 && p->catcolumns && p->catcolumnfile==NULL)
+    error(EXIT_SUCCESS, 0, "WARNING: '--catcolumns' (or '-C') is ignored "
+          "because '--catcolumnfile' (or '-L') was not given to specify "
+          "which file the new columns should come from. To suppress this "
+          "warning, please run with '--quiet'");
+
+  /* Check the floating point format string. */
+  if(p->txtf32fmtstr)
+    {
+      p->txtf32format=gal_table_displayflt_from_str(p->txtf32fmtstr);
+      if(p->txtf32format==GAL_TABLE_DISPLAY_FMT_INVALID)
+        error(EXIT_FAILURE, 0, "'%s' is not a recognized value for "
+              "'--txtf64format'. Recognized values are 'fixed' and "
+              "'exp'", p->txtf32fmtstr);
+    }
+  if(p->txtf64fmtstr)
+    {
+      p->txtf64format=gal_table_displayflt_from_str(p->txtf64fmtstr);
+      if(p->txtf64format==GAL_TABLE_DISPLAY_FMT_INVALID)
+        error(EXIT_FAILURE, 0, "'%s' is not a recognized value for "
+              "'--txtf64format'. Recognized values are 'fixed' and "
+              "'exp'", p->txtf64fmtstr);
+    }
+
+  /* If the user wants easy-to-read text output (this should over-write the
+     existing values in the command-line or in configuration
+     files). Because default values exist in the configuration files. */
+  if(p->txteasy)
+    {
+      p->txtf32precision=3;
+      p->txtf64precision=6;
+      p->txtf32format=GAL_TABLE_DISPLAY_FMT_FIXED;
+      p->txtf64format=GAL_TABLE_DISPLAY_FMT_FIXED;
+    }
 }
 
 
@@ -440,9 +518,9 @@ ui_list_select_free(struct list_select *list, int freevalue)
 /**************************************************************/
 /***************      Packaged columns      *******************/
 /**************************************************************/
-/* Return the last outcols element. */
+/* Return the last 'colpack' element. */
 static struct column_pack *
-ui_outcols_last(struct column_pack *list)
+ui_colpack_last(struct column_pack *list)
 {
   if(list)
     {
@@ -459,7 +537,7 @@ ui_outcols_last(struct column_pack *list)
 /* Allocate a clean 'out_columns' structure and put it at the top of the
    list. */
 static struct column_pack *
-ui_outcols_add_new_to_end(struct column_pack **list)
+ui_colpack_add_new_to_end(struct column_pack **list)
 {
   struct column_pack *last, *node;
 
@@ -480,7 +558,7 @@ ui_outcols_add_new_to_end(struct column_pack **list)
      add this node. */
   if(*list)
     {
-      last=ui_outcols_last(*list);
+      last=ui_colpack_last(*list);
       last->next=node;
     }
   else
@@ -494,8 +572,8 @@ ui_outcols_add_new_to_end(struct column_pack **list)
 
 
 
-static void
-ui_outcols_free(struct column_pack *list)
+void
+ui_colpack_free(struct column_pack *list)
 {
   struct column_pack *tmp;
   while(list!=NULL)
@@ -529,45 +607,54 @@ ui_outcols_free(struct column_pack *list)
 /**************************************************************/
 /***************       Preparations         *******************/
 /**************************************************************/
+  /* Read the table metadata to print information. */
+static gal_data_t *
+ui_info_read(struct tableparams *p, size_t *numcols, size_t *numrows,
+             int *tableformat)
+{
+  gal_data_t *allcols;
+  gal_list_str_t *lines;
+  lines=gal_options_check_stdin(p->filename, p->cp.stdintimeout, "input");
+  allcols=gal_table_info(p->filename, p->cp.hdu, lines, numcols,
+                         numrows, tableformat, "--hdu");
+  if(p->filename==NULL) p->filename="Standard-input";
+  gal_list_str_free(lines, 1);
+  return allcols;
+}
+
+
+
+
+
+/* Print full column and row information. */
 static void
 ui_print_info_exit(struct tableparams *p)
 {
   char *tmp;
   int tableformat;
   gal_data_t *allcols;
-  gal_list_str_t *lines;
   size_t numcols, numrows;
 
-  /* Read the table information for the number of columns and rows. */
-  lines=gal_options_check_stdin(p->filename, p->cp.stdintimeout, "input");
-  allcols=gal_table_info(p->filename, p->cp.hdu, lines, &numcols,
-                         &numrows, &tableformat);
-  if(p->filename==NULL) p->filename="Standard-input";
-  gal_list_str_free(lines, 1);
+  /* Read the input. */
+  allcols=ui_info_read(p, &numcols, &numrows, &tableformat);
 
   /* If there was no actual data in the file, then inform the user */
   if(allcols==NULL)
     error(EXIT_FAILURE, 0, "%s: no usable data rows", p->filename);
 
-
   /* Print the file information. */
   printf("--------\n");
   tmp=gal_fits_name_save_as_string(p->filename, p->cp.hdu);
   printf("%s\n", tmp);
   free(tmp);
 
-
   /* Print each column's information. */
   gal_table_print_info(allcols, numcols, numrows);
 
-
   /* Free the information from all the columns. */
   gal_data_array_free(allcols, numcols, 0);
 
-
-  /* Free the allocated spaces and exit. Otherwise, add the number of
-     columns to the list if the user wanted to print the columns
-     (didn't just want their information. */
+  /* Free the allocated spaces and exit. */
   ui_free_report(p);
   exit(EXIT_SUCCESS);
 }
@@ -576,112 +663,204 @@ ui_print_info_exit(struct tableparams *p)
 
 
 
+/* Print number of rows or columns */
+static void
+ui_print_info_nums_exit(struct tableparams *p)
+{
+  int tableformat;
+  gal_data_t *allcols;
+  size_t numcols, numrows;
+
+  /* Read the input metadata and free the column information (which is not
+     needed here: we just want the numbers). */
+  allcols=ui_info_read(p, &numcols, &numrows, &tableformat);
+  gal_data_array_free(allcols, numcols, 0);
+
+  /* Print the respective information. */
+  if(p->infonumcols) printf("%zu\n", numcols);
+  else if(p->infonumrows) printf("%zu\n", numrows);
+  else
+    error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at '%s' to fix "
+          "the problem. This function should only be activated when "
+          "'--info-num-cols' or '--info-num-rows' are called", __func__,
+          PACKAGE_BUGREPORT);
+
+  /* Free the allocated spaces and exit. */
+  ui_free_report(p);
+  exit(EXIT_SUCCESS);
+}
+
+
+
+
+
+static void
+ui_columns_prepare_arith(struct tableparams *p, gal_data_t *colinfo,
+                         gal_list_str_t **colstoread, size_t *totcalled,
+                         size_t numcols, char *str)
+{
+  struct column_pack *node;
+
+  /* If this is the first arithmetic operation and the user has
+     already asked for some columns, we'll need to put all
+     previously requested simply-printed columns into an 'colpack'
+     structure, then add this arithmetic operation's 'colpack'. */
+  if(p->colpack==NULL && *colstoread)
+    {
+      /* Allocate an empty structure and set the necessary
+         pointers. */
+      node=ui_colpack_add_new_to_end(&p->colpack);
+      node->start=0;
+      node->numsimple=gal_list_str_number(*colstoread);
+      *totcalled=node->numsimple;
+    }
+
+  /* Add a new column pack for this arithmetic operation, then read
+     all the tokens (while specifying which columns it needs). */
+  node=ui_colpack_add_new_to_end(&p->colpack);
+  arithmetic_init(p, &node->arith, colstoread, totcalled,
+                  str+ARITHMETIC_CALL_LENGTH, colinfo, numcols);
+}
+
+
+
+
+
 /* The columns can be given as comma-separated values to one option or
    multiple calls to the column option. Here, we'll check if the input list
    has comma-separated values. If they do then the list will be updated to
    be fully separate. */
 static void
-ui_columns_prepare(struct tableparams *p)
+ui_columns_prepare(struct tableparams *p, gal_list_str_t *lines)
 {
-  gal_data_t *strs;
-  char *c, **strarr;
-  size_t i, totcalled=0;
+  char *tstr;
+  gal_data_t *colinfo=NULL;
+  int tableformat, arithallind=0;
   struct column_pack *node, *last;
-  gal_list_str_t *tmp, *toread=NULL;
+  gal_list_str_t *tmp, *colstoread=NULL;
+  size_t i, totcalled=0, numcols, numrows, numsimple;
+  char *c, *str, countstr[11]; /* an un-signed 32-bit integer takes 10 chars */
 
-  /* Go over the whole original list (where each node may have more than
-     one value separated by a comma. */
+  /* Go over the list of requested columns from the main input. */
   for(tmp=p->columns;tmp!=NULL;tmp=tmp->next)
     {
-      /* Remove any possibly commented new-line where we have a backslash
-         followed by a new-line character (replace the two characters with
-         two single space characters). This can happen with the 'arith'
-         argument in a script, when it gets long (bug #58371). But to be
-         general in other cases too, we'll just correct it here. */
-      for(c=tmp->v;*c!='\0';++c)
-        if(*c=='\\' && *(c+1)=='\n') { *c=' '; *(++c)=' '; }
+      /* For easy reading. */
+      str=tmp->v;
 
-      /* Read the different comma-separated strings into an array (within a
-         'gal_data_t'). */
-      strs=gal_options_parse_csv_strings_raw(tmp->v, NULL, 0);
-      strarr=strs->array;
-
-      /* Go over all the given colum names/numbers. */
-      for(i=0;i<strs->size;++i)
+      /* If this is an arithmetic column.  */
+      if(!strncmp(str, ARITHMETIC_CALL, ARITHMETIC_CALL_LENGTH))
         {
-          /* See if this is an arithmetic column to be processed, or the
-             contents should just be printed. */
-          if(!strncmp(strarr[i], ARITHMETIC_CALL, ARITHMETIC_CALL_LENGTH))
-            {
-              /* If this is the first arithmetic operation and the user has
-                 already asked for some columns, we'll need to put all
-                 previously requested simply-printed columns into an
-                 'outcols' structure, then add this arithmetic operation's
-                 'outcols'. */
-              if(p->outcols==NULL && toread)
-                {
-                  /* Allocate an empty structure and set the necessary
-                     pointers. */
-                  node=ui_outcols_add_new_to_end(&p->outcols);
-                  node->start=0;
-                  node->numsimple=gal_list_str_number(toread);
-                  totcalled=node->numsimple;
-                }
+          /* Arithmetic operations may be done on columns from other files
+             (for example with '--catcolumnfile'). We therefore need to
+             check if the requested column is in the main input file or
+             not. If not, it should be set when column arithmetic
+             starts. To do this, we need to get the input's column
+             information. */
+          if(colinfo==NULL)
+            colinfo=gal_table_info(p->filename, p->cp.hdu, lines,
+                                   &numcols, &numrows, &tableformat,
+                                   "--hdu");
 
-              /* Add a new column pack, then read all the tokens (while
-                 specifying which columns it needs). */
-              node=ui_outcols_add_new_to_end(&p->outcols);
-              arithmetic_init(p, &node->arith, &toread, &totcalled,
-                              strarr[i]+ARITHMETIC_CALL_LENGTH);
-              free(strarr[i]);
+          /* Check if '$_all' is in the string. */
+          for(c=str; *c!='\0'; ++c)
+            if(strncmp(c, "$_all", 5)==0)
+              { arithallind=c-str; break; }
+
+          /* When '$_all' is in the string, we need to repeat this option
+             for every column. Otherwise, just add this option once.*/
+          if(arithallind)
+            {
+              /* Because we want to use the same bytes as '$_all' in the
+                 input string, we have four characters to write over '_all'
+                 with the number of each column. However, 'sprintf' already
+                 puts a '\0' on the last character (which we later replace
+                 with a ' '), so we only have three characters to use for
+                 column numbers. Therefore, to use this feature the input
+                 table can have a maximum of 999 columns (the FITS standard
+                 only accepts 999 columns; so it is very rare for people to
+                 need more! but we can add it if necessary). */
+              if(numcols>999)
+                error(EXIT_FAILURE, 0, "the '$_all' feature is currently "
+                      "only implemented for tables with fewer than 999 "
+                      "columns. Please contact us at %s to add more columns",
+                      PACKAGE_BUGREPORT);
+
+              /* Repeat the arithmetic command for each column. */
+              for(i=1;i<=numcols;++i)
+                {
+                  gal_checkset_allocate_copy(str, &tstr);
+                  sprintf(tstr+arithallind+1, "%-3zu", i);
+                  tstr[arithallind+4]=' ';
+                  ui_columns_prepare_arith(p, colinfo, &colstoread, &totcalled,
+                                           numcols, tstr);
+                  free(tstr);
+                }
             }
-          /* This is a simple column (no change in values). */
           else
-            {
-              /* Add this column to the list of columns to read. */
-              gal_list_str_add(&toread, strarr[i], 0);
-
-              /* See if we have packaged the output columns. */
-              if(p->outcols)
-                {
-                  /* If the previous column package was an arithmetic
-                     operation, allocate a new node. */
-                  last=ui_outcols_last(p->outcols);
-                  if(last->arith)
-                    {
-                      node=ui_outcols_add_new_to_end(&p->outcols);
-                      node->start=totcalled;
-                      node->numsimple=1;
-                    }
-
-                  /* The previous package of columns are simple (we don't
-                     need to change their value), so we can just increment
-                     the number of columns there and don't need to allocate
-                     a new one. */
-                  else
-                    last->numsimple+=1;
-                }
-
-              /* Increment the total number of called columns. */
-              totcalled+=1;
-            }
-
-          /* The pointer allocated string is either being used (and later
-             freed) else, or has already been freed. So its necessary to
-             set it to NULL. */
-          strarr[i]=NULL;
+            ui_columns_prepare_arith(p, colinfo, &colstoread, &totcalled,
+                                     numcols, str);
         }
 
-      /* Clean up. */
-      gal_data_free(strs);
+      /* This is a simple column (no change in values). */
+      else
+        {
+          /* If the value is '_all', then we should add all the input's
+             columns. Otherwise, just add this string. */
+          if( !strcmp(str, "_all") )
+            {
+              /* Load column information (it not already loaded). */
+              if(colinfo==NULL)
+                colinfo=gal_table_info(p->filename, p->cp.hdu, lines,
+                                       &numcols, &numrows,
+                                       &tableformat, "--hdu");
+
+              /* Add all the input column counters to the list of columns
+                 to read */
+              numsimple=gal_list_data_number(colinfo);
+              for(i=0;i<numsimple;++i)
+                {
+                  sprintf(countstr, "%u", (uint32_t)(i+1));
+                  gal_list_str_add(&colstoread, countstr, 1);
+                }
+            }
+          else
+            {
+              numsimple=1;
+              gal_list_str_add(&colstoread, str, 1);
+            }
+
+          /* See if we have packaged the output columns. */
+          if(p->colpack)
+            {
+              /* If the previous column package was an arithmetic
+                 operation, allocate a new node. */
+              last=ui_colpack_last(p->colpack);
+              if(last->arith)
+                {
+                  node=ui_colpack_add_new_to_end(&p->colpack);
+                  node->start=totcalled;
+                  node->numsimple=numsimple;
+                }
+
+              /* The previous package of columns are simple (we don't need
+                 to change their value), so we can just increment the
+                 number of columns there and don't need to allocate a new
+                 one. */
+              else
+                last->numsimple+=numsimple;
+            }
+
+          /* Increment the total number of called columns. */
+          totcalled+=1;
+        }
     }
 
   /* For a check
-  if(p->outcols)
+  if(p->colpack)
     {
       struct column_pack *tmp;
       struct arithmetic_token *atmp;
-      for(tmp=p->outcols;tmp!=NULL;tmp=tmp->next)
+      for(tmp=p->colpack;tmp!=NULL;tmp=tmp->next)
         if(tmp->arith)
           {
             printf("Arithmetic: \n");
@@ -705,11 +884,15 @@ ui_columns_prepare(struct tableparams *p)
     }
   */
 
-  /* Delete the old list, then reverse the 'toread' list, and put it into
-     'p->columns'. */
+  /* Free the information from all the columns (that may have been gathered
+     if it was necessary). */
+  if(colinfo) gal_data_array_free(colinfo, numcols, 0);
+
+  /* Delete the old list, then reverse the 'colstoread' list, and put it
+     into 'p->columns'. */
   gal_list_str_free(p->columns, 1);
-  gal_list_str_reverse(&toread);
-  p->columns=toread;
+  gal_list_str_reverse(&colstoread);
+  p->columns=colstoread;
 }
 
 
@@ -825,8 +1008,8 @@ ui_check_select_sort_before(struct tableparams *p, gal_list_str_t *lines,
   if(p->inpolygon)
     {
       strarr=p->inpolygon->array;
-      inpolytmp=gal_data_alloc(NULL, GAL_TYPE_UINT8, 1, &one, NULL, 1, -1, 1,
-                               strarr[0], NULL, NULL);
+      inpolytmp=gal_data_alloc(NULL, GAL_TYPE_UINT8, 1, &one, NULL, 1,
+                               -1, 1, strarr[0], NULL, NULL);
       inpolytmp->next=gal_data_alloc(NULL, GAL_TYPE_UINT8, 1, &one, NULL,
                                      1, -1, 1, strarr[1], NULL, NULL);
       select[SELECT_TYPE_INPOLYGON]=inpolytmp;
@@ -834,8 +1017,8 @@ ui_check_select_sort_before(struct tableparams *p, gal_list_str_t *lines,
   if(p->outpolygon)
     {
       strarr=p->outpolygon->array;
-      outpolytmp=gal_data_alloc(NULL, GAL_TYPE_UINT8, 1, &one, NULL, 1, -1, 1,
-                               strarr[0], NULL, NULL);
+      outpolytmp=gal_data_alloc(NULL, GAL_TYPE_UINT8, 1, &one, NULL,
+                                1, -1, 1, strarr[0], NULL, NULL);
       outpolytmp->next=gal_data_alloc(NULL, GAL_TYPE_UINT8, 1, &one, NULL,
                                      1, -1, 1, strarr[1], NULL, NULL);
       select[SELECT_TYPE_OUTPOLYGON]=outpolytmp;
@@ -844,7 +1027,7 @@ ui_check_select_sort_before(struct tableparams *p, gal_list_str_t *lines,
 
   /* Get all the input table's column information. */
   allcols=gal_table_info(p->filename, p->cp.hdu, lines, &numcols,
-                         &numrows, &tableformat);
+                         &numrows, &tableformat, "--hdu");
 
 
   /* If '--noblank' is given an '_all', we should replace it with the
@@ -1012,7 +1195,8 @@ ui_check_select_sort_before(struct tableparams *p, gal_list_str_t *lines,
       for(k=0;k<SELECT_TYPE_NUMBER;++k)
         for(dtmp=select[k];dtmp!=NULL;dtmp=dtmp->next)
           {
-            if(*sortindout!=GAL_BLANK_SIZE_T && selectindout[i]==*sortindout)
+            if(  *sortindout!=GAL_BLANK_SIZE_T
+               && selectindout[i]==*sortindout)
               {
                 selecttypeout[i]=k;
                 selectindout[i]=*sortindout;
@@ -1080,6 +1264,13 @@ ui_check_select_sort_after(struct tableparams *p, size_t nselect,
     }
 
 
+  /* The column to sort by should not be a vector column. */
+  if(p->sortcol && p->sortcol->ndim!=1)
+    error(EXIT_FAILURE, 0, "the column given to '--sort' cannot be a "
+          "vector column. If you need this feature, please get in "
+          "touch with us at '%s' to add it", PACKAGE_BUGREPORT);
+
+
   /* Since we can have several selection columns, we'll treat them
      differently. */
   for(i=0;i<nselect;++i)
@@ -1131,8 +1322,6 @@ ui_check_select_sort_after(struct tableparams *p, size_t nselect,
 static void
 ui_preparations(struct tableparams *p)
 {
-  double *darr;
-  size_t i, *colmatch;
   gal_list_str_t *lines;
   size_t nselect=0, origoutncols=0;
   size_t sortindout=GAL_BLANK_SIZE_T;
@@ -1141,18 +1330,18 @@ ui_preparations(struct tableparams *p)
 
   /* If there were no columns specified or the user has asked for
      information on the columns, we want the full set of columns. */
-  if(p->information)
-    ui_print_info_exit(p);
-
-
-  /* Prepare the column names. */
-  ui_columns_prepare(p);
+  if(p->information)                   ui_print_info_exit(p);
+  if(p->infonumcols || p->infonumrows) ui_print_info_nums_exit(p);
 
 
   /* If the input is from stdin, save it as 'lines'. */
   lines=gal_options_check_stdin(p->filename, p->cp.stdintimeout, "input");
 
 
+  /* Prepare the column names. */
+  ui_columns_prepare(p, lines);
+
+
   /* If any kind of row-selection is requested set 'p->selection' to 1. */
   p->selection = ( p->range || p->inpolygon || p->outpolygon || p->equal
                    || p->notequal || p->noblankll );
@@ -1162,22 +1351,24 @@ ui_preparations(struct tableparams *p)
      extra columns. */
   if(p->selection || p->sort)
     ui_check_select_sort_before(p, lines, &nselect, &origoutncols,
-                                &sortindout, &selectindout, &selecttypeout);
+                                &sortindout, &selectindout,
+                                &selecttypeout);
 
 
   /* If we have any arithmetic operations, we need to make sure how many
      columns match every given column name. */
-  colmatch = ( p->outcols
-               ? gal_pointer_allocate(GAL_TYPE_SIZE_T,
-                                      gal_list_str_number(p->columns), 1,
-                                      __func__, "colmatch")
-               : NULL);
+  p->colmatch = ( p->colpack
+                  ? gal_pointer_allocate(GAL_TYPE_SIZE_T,
+                                         gal_list_str_number(p->columns),
+                                         1, __func__, "p->colmatch")
+                  : NULL);
 
 
   /* Read the necessary columns. */
   p->table=gal_table_read(p->filename, cp->hdu, lines, p->columns,
                           cp->searchin, cp->ignorecase, cp->numthreads,
-                          cp->minmapsize, p->cp.quietmmap, colmatch);
+                          cp->minmapsize, p->cp.quietmmap, p->colmatch,
+                          "--hdu");
   if(p->filename==NULL) p->filename="stdin";
   gal_list_str_free(lines, 1);
 
@@ -1196,74 +1387,18 @@ ui_preparations(struct tableparams *p)
           "non-blank lines)", p->filename);
 
 
-  /* Set the final indexs. */
-  if(p->outcols)
-    arithmetic_indexs_final(p, colmatch);
-
-
   /* Make sure the (possible) output name is writable. */
-  gal_checkset_writable_remove(p->cp.output, 0, p->cp.dontdelete);
+  gal_checkset_writable_remove(p->cp.output, p->filename, 0,
+                               p->cp.dontdelete);
 
 
-  /* If the head or tail values are given and are larger than the number of
-     rows, just set them to the number of rows (print the all the final
-     rows). This is how the 'head' and 'tail' programs of GNU Coreutils
-     operate. */
-  p->head = ( ((p->head!=GAL_BLANK_SIZE_T) && (p->head > p->table->size))
-              ? p->table->size
-              : p->head );
-  p->tail = ( ((p->tail!=GAL_BLANK_SIZE_T) && (p->tail > p->table->size))
-              ? p->table->size
-              : p->tail );
-
-  /* If rows are given, do some sanity checks and make sure that they are
-     within the table's limits. */
-  if(p->rowlimit)
-    {
-      /* There should only be two values. */
-      if(p->rowlimit->size!=2)
-        error(EXIT_FAILURE, 0, "only two should be given to "
-              "'--rowlimit' (the top and bottom row numbers specifying "
-              "your desired range)");
-
-      /* Do individual checks. */
-      darr=p->rowlimit->array;
-      for(i=0;i<p->rowlimit->size;++i)
-        {
-          /* Make sure it isn't 0 or negative. */
-          if( darr[i]<=0 )
-            error(EXIT_FAILURE, 0, "%g (value given to '--rowlimit') "
-                  "is smaller than, or equal to, zero! This option's "
-                  "values are row-counters (starting from 1), so they "
-                  "must be positive integers", darr[i]);
-
-          /* Make sure its an integer. */
-          if( darr[i] != (size_t)(darr[i]) )
-            error(EXIT_FAILURE, 0, "%g (value given to '--rowlimit') is "
-                  "not an integer! This option's values are row-counters "
-                  "so they must be integers.", darr[i]);
-
-          /* Subtract 1 from the value, so it counts from 0. */
-          --darr[i];
-        }
-
-      /* Make sure that the first value is smaller than the second. */
-      if( darr[0] > darr[1] )
-        error(EXIT_FAILURE, 0, "the first value to '--rowlimit' (%g) is "
-              "larger than the second (%g). This option's values defines "
-              "a row-counter interval, assuming the first value is the top "
-              "of the desired interval (smaller row counter) and the second "
-              "value is the bottom of the desired interval (larger row "
-              "counter)", darr[0], darr[1]);
-    }
-
   /* If random rows are desired, we need to define a GSL random number
      generator structure. */
   if(p->rowrandom)
     p->rng=gal_checkset_gsl_rng(p->envseed, &p->rng_name, &p->rng_seed);
 
+
   /* Clean up. */
-  free(colmatch);
   if(selectindout) free(selectindout);
   if(selecttypeout) free(selecttypeout);
 }
@@ -1321,9 +1456,8 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct tableparams *p)
   gal_options_read_config_set(&p->cp);
 
 
-  /* Read the options into the program's structure, and check them and
-     their relations prior to printing. */
-  ui_read_check_only_options(p);
+  /* Sanity check only on options. */
+  ui_check_only_options(p);
 
 
   /* Print the option values if asked. Note that this needs to be done
@@ -1384,12 +1518,11 @@ ui_free_report(struct tableparams *p)
   /* Free the allocated arrays: */
   free(p->cp.hdu);
   free(p->cp.output);
-  ui_outcols_free(p->outcols);
   gal_list_data_free(p->table);
   if(p->wcshdu) free(p->wcshdu);
   gal_list_data_free(p->noblank);
   gal_list_str_free(p->columns, 1);
-  if(p->colarray) free(p->colarray);
+  if(p->colmatch) free(p->colmatch);
   gal_list_data_free(p->colmetadata);
   gal_list_str_free(p->catcolumnhdu, 1);
   gal_list_str_free(p->catcolumnfile, 1);
